/*
* From
* Revision 1.13  2020-04-16 12:05:33  abrami
*
*/

/* Necessary includes for device drivers */
#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h> /* printk() */
#include <linux/slab.h> /* kmalloc() */
#include <linux/fs.h> /* everything... */
#include <linux/errno.h> /* error codes */
#include <linux/types.h> /* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h> /* O_ACCMODE */
// include <asm/system.h> /* cli(), *_flags */
#include <asm/uaccess.h> /* copy_from/to_user */
#include <linux/device.h> /* class_creatre */
#include <linux/cdev.h> /* cdev_init */
#include <linux/poll.h>
#include <linux/vmalloc.h>

#include <linux/mutex.h>

#include <linux/version.h>

#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 27)
	#include <linux/sched.h>
	#include <linux/wait.h>
#endif

#include <linux/string.h>
#include <linux/delay.h>
#include <linux/version.h>

//---------------------------------------------------------------------------

#ifdef _WQMEMCPY_
#include <linux/workqueue.h>


struct work_memcpy {
	struct work_struct work_s;
	size_t count;
};
#endif

//---------------------------------------------------------------------------
#ifdef __GNUC__
	#define VARIABLE_IS_NOT_USED __attribute__ ((unused))
#else
	#define VARIABLE_IS_NOT_USED
#endif

#define VNOTU VARIABLE_IS_NOT_USED
//---------------------------------------------------------------------------

#define DEVICE_NAME "pesp"

MODULE_LICENSE("Dual BSD/GPL");

#define MAX_FD_NR 5

/* Buffer to store data */
struct _buff{
	char *buff;
	atomic_t st[MAX_FD_NR];
};


typedef struct {
	pid_t pid;
	char comm[TASK_COMM_LEN];
	int opened;
	char *buff;
	atomic_t st;
	struct mutex mtx;
	int want_update;
	size_t wr_count;
} buff_t;

typedef struct {
	atomic_t inited;
	wait_queue_head_t queue;
	buff_t buff[MAX_FD_NR];
} pespBuff_t;

#define WANT_UPDATE_DFLT 1

//---------------------------------------------------------------------------
typedef struct {
	int opened;
	atomic_t st;
} buff_Dt_t;

/* Declaration of memory.c functions */
int __init pesp_init(void);
void __exit pesp_exit(void);

int          pesp_open(struct inode *inode, struct file *filp);
int          pesp_release(struct inode *inode, struct file *filp);
ssize_t      pesp_read(struct file *filp, char *buf, size_t count, loff_t *f_pos);
ssize_t      pesp_write(struct file *filp, const char *buf, size_t count, loff_t *f_pos);
unsigned int pesp_poll(struct file *filp, poll_table *pt);

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,32)
	int 	pesp_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg);
#else
	long 	pesp_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);
#endif

#ifdef _PROCFS_
	#define CLIBUFFSIZE 512
	#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,32)
		int	pesp_rproc_dev(char *buf, char **start, off_t offset, int count, int *eof, void *data);
		int	pesp_rproc_tasks(char *buf, char **start, off_t offset, int count, int *eof, void *data);

		int	pesp_rproc_gc_process(char *buf, char **start, off_t offset, int count, int *eof, void *data);
		int	pesp_wproc_gc_process(struct file *file, const char *buf, int count, void *data );

		int	pesp_rproc_gc_qf_flag(char *buf, char **start, off_t offset, int count, int *eof, void *data);
		int	pesp_wproc_gc_qf_flag(struct file *file, const char *buf, int count, void *data );

		int	pesp_rproc_cli(char *buf, char **start, off_t offset, int count, int *eof, void *data);
		int	pesp_wproc_cli(struct file *file, const char *buf, int count, void *data );

	#else
		int	pesp_rproc_dev(struct seq_file *m, void *v);
		int	pesp_rproc_tasks(struct seq_file *m, void *v);

		int	pesp_rproc_gc_in_params(struct seq_file *m, void *v);

		int 	pesp_rproc_gc_process(struct seq_file *m, void *v);
		ssize_t	pesp_wproc_gc_process(struct file *file, const char * buf, size_t size, loff_t * ppos);

		int 	pesp_rproc_gc_qf_flag(struct seq_file *m, void *v);
		ssize_t	pesp_wproc_gc_qf_flag(struct file *file, const char * buf, size_t size, loff_t * ppos);

		int	pesp_rproc_cli(struct seq_file *m, void *v);
		ssize_t pesp_wproc_cli(struct file *file, const char * buf, size_t size, loff_t * ppos);
	#endif
#endif

#include "pesp_ioctl.h"

