#include "pesp.h"


//---------------------------------------------------------------------------
extern h_virt_ccd_t pesp;
extern struct _pespImg pespDark;
extern atomic_t pespcmd;
extern h_ccd_t  pespCCD;
extern stageInfo_t stageInfo;
extern float pixel2micron;

extern ccd_point_t OYag;

#ifdef _HAVE_GC_
	extern gc_in_params_t gc_in_params;
#endif

extern pespBuff_t pespBuff;

extern struct timeval ptimers[MAX_FD_NR][5];

extern h_metaimg_t dark_buf;

extern long int buf_size;

extern char wclibuf[CLIBUFFSIZE];
extern char rclibuf[CLIBUFFSIZE];

//---------------------------------------------------------------------------
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,32)
	int pesp_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg){
#else
	long pesp_ioctl(struct file *filp, unsigned int cmd, unsigned long arg){
#endif

	int err = 0;
	//int tmp;
	int retval;
	int size;
	int v;
	unsigned long w;
  	int ch = -1;
	two_int_t twoint;

 	unsigned int dev_minor = iminor(filp->f_path.dentry->d_inode);

	if ( dev_minor == 0 ){   // pespImg0, vecchio device
  		ch = (uintptr_t)filp->private_data;
	};

	retval = -ENOTTY;
    
	/*
	 * extract the type and number bitfields, and don't decode
	 * wrong cmds: return ENOTTY (inappropriate ioctl) before access_ok()
	 */
	if (_IOC_TYPE(cmd) != PESP_IOC_MAGIC) return -ENOTTY;
	if (_IOC_NR(cmd) > PESP_IOC_MAXNR) return -ENOTTY;

	/*
	 * the direction is a bitmask, and VERIFY_WRITE catches R/W
	 * transfers. `Type' is user-oriented, while
	 * access_ok is kernel-oriented, so the concept of "read" and
	 * "write" is reversed
	 */
	if (_IOC_DIR(cmd) & _IOC_READ)
		err = !access_ok(VERIFY_WRITE, (void __user *)arg, _IOC_SIZE(cmd));
	else if (_IOC_DIR(cmd) & _IOC_WRITE)
		err =  !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));
	if (err) return -EFAULT;

	switch(cmd) {

	  case PESP_IOCS_AFR:
		  size = sizeof(float);
		  v = copy_from_user(&(pesp.acqframerate), (float __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_DARK:
		  {
		    h_metaimg_t dt;
		    v = copy_from_user(&dt, (h_metaimg_t __user *)arg, sizeof(h_metaimg_t));

  		    printk(KERN_DEBUG "pesp():  PESP_IOCG_DARK buf_size = %ld dt.info.size = %d\n", buf_size, dt.info.size);

		    //if (dt.info.size > buf_size) {retval = -ENOMEM; break;}
		    if (dt.info.size < dark_buf.info.size) {retval = -ENOBUFS; break;}

  		    printk(KERN_DEBUG "pesp():  PESP_IOCG_DARK dt.info.size = %d\n", dt.info.size);

		    dt.info = dark_buf.info;
  		    printk(KERN_DEBUG "pesp():  PESP_IOCG_DARK going to cpy = %d bytes\n", dark_buf.info.size - dark_buf.info.info_size);
		    v = copy_to_user((unsigned char __user *)dt.data, dark_buf.data, dark_buf.info.size - dark_buf.info.info_size);
		    if ( v == 0 ) {
		        v = copy_to_user((data_t __user *)arg, &dt, sizeof(h_metaimg_t));
			retval = 0;
		    } else retval = -EFAULT;
		  }
		break;

	  case PESP_IOCS_DARK:
		  {
		    h_metaimg_t dt;
		    v = copy_from_user(&dt, (h_metaimg_t __user *)arg, sizeof(h_metaimg_t));
		    if (dt.info.size <= 0) {retval = -EBADR; break;}
		    if (dt.info.size >= buf_size) {retval = -EFBIG; break;}

  		    printk(KERN_DEBUG "pesp():  PESP_IOCS_DARK dt.info.size = %d\n", dt.info.size);
  		    printk(KERN_DEBUG "pesp():  PESP_IOCS_DARK dt.data = %p\n", dt.data);
		    v = copy_from_user(dark_buf.data, (unsigned char __user *)dt.data, dt.info.size - dt.info.info_size);
		    if ( v == 0 ) {
			dark_buf.info = dt.info;
			retval = 0;
		    } else retval = -EFAULT;
		  }
		break;

	  case PESP_IOCG_CMD:
		  w = atomic_read( &pespcmd );
		  retval = __put_user(w, (unsigned long __user *)arg);
		break;

	  case PESP_IOCT_CMD:
  		  atomic_set( &pespcmd, (unsigned long)arg);
		  retval = 0;
		break;

	  case PESP_IOCG_VCCD:
		  size = sizeof(h_virt_ccd_t);
		  v = copy_to_user((h_virt_ccd_t __user *)arg, &pesp, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCS_VCCD:
		  size = sizeof(h_virt_ccd_t);
		  v = copy_from_user(&pesp, (h_virt_ccd_t __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_CCD:
		  size = sizeof(h_ccd_t );
		  v = copy_to_user((h_ccd_t  __user *)arg, &pespCCD, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCS_CCD:
		  size = sizeof(h_ccd_t );
		  v = copy_from_user(&pespCCD, (h_ccd_t  __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_AET:
		// BBBB  retval = __put_user(pespCCD.aet, (struct h_timeval __user *)arg);
		  size = sizeof(struct h_timeval);
		  v = copy_to_user((struct h_timeval __user *)arg, &(pespCCD.aet), size);
		break;

	  case PESP_IOCS_AET:
		  size = sizeof(struct h_timeval);
		  v = copy_from_user(&(pespCCD.aet), (struct h_timeval __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_AMD:
		retval = __put_user(pespCCD.amd, (char __user *)arg);
		break;

	  case PESP_IOCS_AMD:
		retval = __get_user(pespCCD.amd, (char __user *)arg);
		break;

	  case PESP_IOCG_EMD:
		retval = __put_user(pespCCD.emd, (char __user *)arg);
		break;

	  case PESP_IOCS_EMD:
		retval = __get_user(pespCCD.emd, (char __user *)arg);
		break;

	  case PESP_IOCG_SMD:
		retval = __put_user(pespCCD.smd, (char __user *)arg);
		break;

	  case PESP_IOCS_SMD:
		retval = __get_user(pespCCD.smd, (char __user *)arg);
		break;

	  case PESP_IOCG_ATP:
		retval = __put_user(pespCCD.atp, (char __user *)arg);
		break;

	  case PESP_IOCS_ATP:
		retval = __get_user(pespCCD.atp, (char __user *)arg);
		break;

	  case PESP_IOCG_ESC:
		retval = __put_user(pespCCD.esc, (char __user *)arg);
		break;

	  case PESP_IOCS_ESC:
		retval = __get_user(pespCCD.esc, (char __user *)arg);
		break;

	  case PESP_IOCG_CEG:
		retval = __put_user(pespCCD.ceg, (unsigned short int __user *)arg);
		break;

	  case PESP_IOCS_CEG:
		retval = __get_user(pespCCD.ceg, (unsigned short int __user *)arg);
		break;

	  case PESP_IOCG_CEO:
		retval = __put_user(pespCCD.ceo, (unsigned short int __user *)arg);
		break;

	  case PESP_IOCS_CEO:
		retval = __get_user(pespCCD.ceo, (unsigned short int __user *)arg);
		break;

	  case PESP_IOCG_CSTATUS:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				retval = __put_user(pespCCD.priv_data.andor.cstatus, (unsigned long __user *)arg);
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

	  case PESP_IOCS_CSTATUS:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				retval = __get_user(pespCCD.priv_data.andor.cstatus, (unsigned long __user *)arg);
				break;
			default:
				retval = -EINVAL;
				break;
		}

		break;

	  case PESP_IOCG_TEMP:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				size = sizeof(float);
                  		v = copy_to_user((float __user *)arg, &(pespCCD.priv_data.andor.temp), size);
                  		if ( v == 0 ) retval = 0;
                  		else retval = -EFAULT;
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

	  case PESP_IOCS_TEMP:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				size = sizeof(float);
		  		v = copy_from_user(&(pespCCD.priv_data.andor.temp), (float __user *)arg, size);
                  		if ( v == 0 ) retval = 0;
                  		else retval = -EFAULT;
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

	  case PESP_IOCG_DOEXTRA:
		retval = __put_user(pespCCD.do_extra, (short int __user *)arg);
		break;

	  case PESP_IOCS_DOEXTRA:
		retval = __get_user(pespCCD.do_extra, (short int __user *)arg);
		break;

	  case PESP_IOCG_WTEMP:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				retval = __put_user(pespCCD.priv_data.andor.target_temp, (int __user *)arg);
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

	  case PESP_IOCS_WTEMP:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				retval = __get_user(pespCCD.priv_data.andor.target_temp, (int __user *)arg);
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

	  case PESP_IOCG_FASTTRIGGER:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				retval = __put_user(pespCCD.priv_data.andor.fast_trigger, (bool __user *)arg);
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

	  case PESP_IOCS_FASTTRIGGER:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				retval = __get_user(pespCCD.priv_data.andor.fast_trigger, (int __user *)arg);
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

	  case PESP_IOCG_VSSPEED:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				retval = __put_user(pespCCD.priv_data.andor.vsspeed, (unsigned short int __user *)arg);
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

	  case PESP_IOCS_VSSPEED:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				retval = __get_user(pespCCD.priv_data.andor.vsspeed, (unsigned short int __user *)arg);
				break;
			default:
				retval = -EINVAL;
				break;
		}

		break;

	  case PESP_IOCG_READOUTTIME:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				size = sizeof(float);
                  		v = copy_to_user((float __user *)arg, &(pespCCD.priv_data.andor.readouttime), size);
                  		if ( v == 0 ) retval = 0;
                  		else retval = -EFAULT;
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

	  case PESP_IOCS_READOUTTIME:
		switch (pespCCD.ccdtype){
			case A_IKONM934:
				size = sizeof(float);
		  		v = copy_from_user(&(pespCCD.priv_data.andor.readouttime), (float __user *)arg, size);
                  		if ( v == 0 ) retval = 0;
                  		else retval = -EFAULT;
				break;
			default:
				retval = -EINVAL;
				break;
		}
		break;

/***************************
PESP_IOCG_WAVELENGTH       _IOR(float)
PESP_IOCS_WAVELENGTH       _IOW(float)
PESP_IOCG_WAVELENGTHASPAN  _IOR(float)
PESP_IOCS_WAVELENGTHASPAN  _IOW(float)
PESP_IOCG_STAGEINFO        _IOR(struct _stageInfo)
PESP_IOCS_STAGEINFO        _IOW(struct _stageInfo)
***************************/

	  case PESP_IOCG_STAGEINFO:
		  size = sizeof(stageInfo_t);
		  v = copy_to_user((stageInfo_t __user *)arg, &stageInfo, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCS_STAGEINFO:
		  size = sizeof(stageInfo_t);
		  v = copy_from_user(&stageInfo, (stageInfo_t __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCS_WAVELENGTH:
		  size = sizeof(float);
		  v = copy_from_user(&(stageInfo.wavelength), (float __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_WAVELENGTH:
		  size = sizeof(float);
		  v = copy_to_user((float __user *)arg, &(stageInfo.wavelength), size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCS_WAVELENGTHSPAN:
		  size = sizeof(float);
		  v = copy_from_user(&(stageInfo.wavelengthspan), (float __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_WAVELENGTHSPAN:
		  size = sizeof(float);
		  v = copy_to_user((float __user *)arg, &(stageInfo.wavelengthspan), size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCS_PIXEL2MICRON:
		  size = sizeof(float);
		  v = copy_from_user(&(pixel2micron), (float __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_PIXEL2MICRON:
		  size = sizeof(float);
		  v = copy_to_user((float __user *)arg, &(pixel2micron), size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_OYAG:
		  size = sizeof(ccd_point_t);
		  v = copy_to_user((ccd_point_t __user *)arg, &OYag, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCS_OYAG:
		  size = sizeof(ccd_point_t);
		  v = copy_from_user(&OYag, (ccd_point_t __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

//============================================================================

	  case PESP_IOCG_WANTUPDATE:
		if ( ch < 0 ) return -EACCES;
		retval = __put_user(pespBuff.buff[ch].want_update, (int __user *)arg);
		break;

	  case PESP_IOCS_WANTUPDATE:
		if ( ch < 0 ) return -EACCES;
		retval = __get_user(pespBuff.buff[ch].want_update, (int __user *)arg);
		break;

	  case PESP_IOCG_PTIMERS:
		if ( ch < 0 ) return -EACCES;
		{
		  ptimer_t pt;
		  size = sizeof(ptimer_t);
		  pt.which = ((ptimer_t __user *)arg)->which;
		  pt.t.tv_sec  = ptimers[ch][pt.which].tv_sec;
		  pt.t.tv_usec = ptimers[ch][pt.which].tv_usec;
		  v = copy_to_user((ptimer_t __user *)arg, &pt, size);
		}
		break;

	  case PESP_IOCS_PTIMERS:
		if ( ch < 0 ) return -EACCES;
		{
		  ptimer_t pt;
		  size = sizeof(ptimer_t);
		  v = copy_from_user(&pt, (ptimer_t __user *)arg, size);
		  ptimers[ch][pt.which].tv_sec = pt.t.tv_sec;
		  ptimers[ch][pt.which].tv_usec = pt.t.tv_usec;
		}
		break;

//============================================================================
#ifdef _HAVE_GC_
	  case PESP_IOCG_GC_PARAMS:
		  size = sizeof(gc_in_params_t);
		  v = copy_to_user((gc_in_params_t __user *)arg, &gc_in_params, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCS_GC_PARAMS:
		  size = sizeof(gc_in_params_t);
		  v = copy_from_user(&gc_in_params, (gc_in_params_t __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_GC_PROCESS:
		retval = __put_user(gc_in_params.gc_roi.process, (int __user *)arg);
		break;

	  case PESP_IOCS_GC_PROCESS:
		retval = __get_user(gc_in_params.gc_roi.process, (int __user *)arg);
		break;

	  case PESP_IOCG_GC_FASTCALC:
		#ifndef IMGPROCNEW
			retval = -EFAULT;
		#else /* IMGPROCNEW */
			twoint.v[0] = gc_in_params.hor_fast_calc_inc;
			twoint.v[1] = gc_in_params.ver_fast_calc_inc;
			size = sizeof(two_int_t);
			v = copy_to_user((two_int_t __user *)arg, &twoint, size);
		  	if ( v == 0 ) retval = 0;
		  	else retval = -EFAULT;
		#endif /* IMGPROCNEW */
		break;

	  case PESP_IOCS_GC_FASTCALC:
		#ifndef IMGPROCNEW
			retval = -EFAULT;
		#else /* IMGPROCNEW */
			size = sizeof(two_int_t);
		  	v = copy_from_user(&twoint, (two_int_t __user *)arg, size);
		  	if ( v == 0 ){
				retval = 0;
				gc_in_params.hor_fast_calc_inc = twoint.v[0];
				gc_in_params.ver_fast_calc_inc = twoint.v[1];
		  	} else retval = -EFAULT;
		#endif /* IMGPROCNEW */
		break;

	  case PESP_IOCG_GC_ALGORITHM:
		#ifndef IMGPROCNEW
			retval = -EFAULT;
		#else /* IMGPROCNEW */
			twoint.v[0] = gc_in_params.hor_algorithm;
			twoint.v[1] = gc_in_params.ver_algorithm;
			size = sizeof(two_int_t);
			v = copy_to_user((two_int_t __user *)arg, &twoint, size);
		  	if ( v == 0 ) retval = 0;
		  	else retval = -EFAULT;
		#endif /* IMGPROCNEW */
		break;

	  case PESP_IOCS_GC_ALGORITHM:
		#ifndef IMGPROCNEW
			retval = -EFAULT;
		#else /* IMGPROCNEW */
			size = sizeof(two_int_t);
		  	v = copy_from_user(&twoint, (two_int_t __user *)arg, size);
		  	if ( v == 0 ){
				retval = 0;
				gc_in_params.hor_algorithm = twoint.v[0];
				gc_in_params.ver_algorithm = twoint.v[1];
		  	} else retval = -EFAULT;
		#endif /* IMGPROCNEW */
		break;

	  case PESP_IOCG_GC_ROITYPE:
		retval = __put_user(gc_in_params.gc_roi.type, (int __user *)arg);
		break;

	  case PESP_IOCS_GC_ROITYPE:
		retval = __get_user(gc_in_params.gc_roi.type, (int __user *)arg);
		break;

	  case PESP_IOCS_GC_ROITHRESHOLD:
		  size = sizeof(double);
		  v = copy_from_user(&(gc_in_params.gc_roi.threshold), (double __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_GC_ROITHRESHOLD:
		  size = sizeof(double);
		  v = copy_to_user((double __user *)arg, &(gc_in_params.gc_roi.threshold), size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCS_GC_ROISEARCHBCKG:
		  size = sizeof(two_double_t);
  printk(KERN_DEBUG "pesp():  PESP_IOCS_GC_ROISEARCHBCKG\n");
		  v = copy_from_user(&(gc_in_params.gc_roi.search_background_param), (two_double_t __user *)arg, size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_GC_ROISEARCHBCKG:
		  size = sizeof(two_double_t);
		  v = copy_to_user((two_double_t __user *)arg, &(gc_in_params.gc_roi.search_background_param), size);
		  if ( v == 0 ) retval = 0;
		  else retval = -EFAULT;
		break;

	  case PESP_IOCG_GC_CURTHRES:
		retval = __put_user(gc_in_params.cur_thres, (int __user *)arg);
		break;

	  case PESP_IOCS_GC_CURTHRES:
		retval = __get_user(gc_in_params.cur_thres, (int __user *)arg);
		break;

#endif /* _HAVE_GC_ */

	  case PESP_IOCG_CLIREAD:
		  size = 512;
		  v = copy_to_user((char __user *)arg, wclibuf, size);
		  if ( v == 0 ) {
			retval = 0;
			//memset(rclibuf, 0, CLIBUFFSIZE);
		  } else retval = -EFAULT;
		break;

	  case PESP_IOCS_CLIWRITE:
		  size = 512;
		  v = copy_from_user(rclibuf, (char __user *)arg, size);
		  if ( v == 0 ) {
			retval = 0;
			memset(wclibuf, 0, CLIBUFFSIZE);
		  } else retval = -EFAULT;
		break;

	  default:  /* redundant, as cmd was checked against MAXNR */
		return -ENOTTY;
	}

	return retval;
};
