/*
 * Ioctl definitions
 */

#ifndef __PESP_IOTCL_H__
#define __PESP_IOTCL_H__


#ifndef __KERNEL__ 
#include <sys/time.h>
#include <inttypes.h>
#endif

#include <stdbool.h>

#include "h_types_2.h"
#include "h_image_2.h"

//buf_size
#define DEFAULT_PESP_BUF_SIZE	2042000

typedef struct {
	unsigned char * data;
	size_t size;
} data_t;


typedef struct {
	int which;
	struct timeval t;
} ptimer_t;

/* Use '8' as magic number */
#define PESP_IOC_MAGIC  '8'

#define PESP_IOCRESET    	_IO(PESP_IOC_MAGIC, 0)

/*
 * S means "Set" through a ptr,
 * T means "Tell" directly with the argument value
 * G means "Get": reply by setting through a pointer
 * Q means "Query": response is on the return value
 * X means "eXchange": switch G and S atomically
 * H means "sHift": switch T and Q atomically
 */

#define PESP_IOCG_CCD	_IOR(PESP_IOC_MAGIC,   1, h_ccd_t)
#define PESP_IOCS_CCD	_IOW(PESP_IOC_MAGIC,   2, h_ccd_t)
#define PESP_IOCG_AET 		_IOR(PESP_IOC_MAGIC,   3, struct h_timeval)
#define PESP_IOCS_AET 		_IOW(PESP_IOC_MAGIC,   4, struct h_timeval)
#define PESP_IOCG_AMD 		_IOR(PESP_IOC_MAGIC,   5, char)
#define PESP_IOCS_AMD 		_IOW(PESP_IOC_MAGIC,   6, char)
#define PESP_IOCG_EMD 		_IOR(PESP_IOC_MAGIC,   7, char)
#define PESP_IOCS_EMD 		_IOW(PESP_IOC_MAGIC,   8, char)
#define PESP_IOCG_SMD 		_IOR(PESP_IOC_MAGIC,   9, char)
#define PESP_IOCS_SMD 		_IOW(PESP_IOC_MAGIC,  10, char)
#define PESP_IOCG_ATP 		_IOR(PESP_IOC_MAGIC,  11, char)
#define PESP_IOCS_ATP 		_IOW(PESP_IOC_MAGIC,  12, char)
#define PESP_IOCG_ESC 		_IOR(PESP_IOC_MAGIC,  13, char)
#define PESP_IOCS_ESC 		_IOW(PESP_IOC_MAGIC,  14, char)
#define PESP_IOCG_CEG 		_IOR(PESP_IOC_MAGIC,  15, unsigned short int)
#define PESP_IOCS_CEG 		_IOW(PESP_IOC_MAGIC,  16, unsigned short int)
#define PESP_IOCG_CEO 		_IOR(PESP_IOC_MAGIC,  17, unsigned short int)
#define PESP_IOCS_CEO 		_IOW(PESP_IOC_MAGIC,  18, unsigned short int)
#define PESP_IOCG_ADS 		_IOR(PESP_IOC_MAGIC,  19, unsigned short int)
#define PESP_IOCS_ADS 		_IOW(PESP_IOC_MAGIC,  20, unsigned short int)
#define PESP_IOCG_SPX 		_IOR(PESP_IOC_MAGIC,  21, unsigned short int)
#define PESP_IOCS_SPX 		_IOW(PESP_IOC_MAGIC,  22, unsigned short int)
#define PESP_IOCG_SVO 		_IOR(PESP_IOC_MAGIC,  23, int)
#define PESP_IOCS_SVO 		_IOW(PESP_IOC_MAGIC,  24, int)
#define PESP_IOCG_SVW 		_IOR(PESP_IOC_MAGIC,  25, int)
#define PESP_IOCS_SVW 		_IOW(PESP_IOC_MAGIC,  26, int)
#define PESP_IOCG_SHO 		_IOR(PESP_IOC_MAGIC,  27, int)
#define PESP_IOCS_SHO 		_IOW(PESP_IOC_MAGIC,  28, int)
#define PESP_IOCG_SHW 		_IOR(PESP_IOC_MAGIC,  29, int)
#define PESP_IOCS_SHW 		_IOW(PESP_IOC_MAGIC,  30, int)
#define PESP_IOCG_RES 		_IOR(PESP_IOC_MAGIC,  31, char)
#define PESP_IOCS_RES 		_IOW(PESP_IOC_MAGIC,  32, char)
#define PESP_IOCG_VCCD		_IOR(PESP_IOC_MAGIC,  33, h_virt_ccd_t)
#define PESP_IOCS_VCCD		_IOW(PESP_IOC_MAGIC,  34, h_virt_ccd_t)
#define PESP_IOCG_CMD	 	_IOR(PESP_IOC_MAGIC,  35, unsigned long)
#define PESP_IOCT_CMD		_IO(PESP_IOC_MAGIC,  36)
#define PESP_IOCG_DARK		_IOR(PESP_IOC_MAGIC,  37, h_metaimg_t)
#define PESP_IOCS_DARK		_IOW(PESP_IOC_MAGIC,  38, h_metaimg_t)
#define PESP_IOCS_AFR 		_IOW(PESP_IOC_MAGIC,  39, float)
#define PESP_IOCG_WAVELENGTH	_IOR(PESP_IOC_MAGIC,  40, float)
#define PESP_IOCS_WAVELENGTH	_IOW(PESP_IOC_MAGIC,  41, float)
#define PESP_IOCG_WAVELENGTHSPAN	_IOR(PESP_IOC_MAGIC,  42, float)
#define PESP_IOCS_WAVELENGTHSPAN	_IOW(PESP_IOC_MAGIC,  43, float)
#define PESP_IOCG_STAGEINFO	_IOR(PESP_IOC_MAGIC,  44, stageInfo_t)
#define PESP_IOCS_STAGEINFO	_IOW(PESP_IOC_MAGIC,  45, stageInfo_t)
#define PESP_IOCG_PIXEL2MICRON	_IOR(PESP_IOC_MAGIC,  46, float)
#define PESP_IOCS_PIXEL2MICRON	_IOW(PESP_IOC_MAGIC,  47, float)
#define PESP_IOCG_OYAG		_IOR(PESP_IOC_MAGIC,  48, ccd_point_t)
#define PESP_IOCS_OYAG		_IOW(PESP_IOC_MAGIC,  49, ccd_point_t)
#define PESP_IOCG_WANTUPDATE 	_IOR(PESP_IOC_MAGIC,  50, int)
#define PESP_IOCS_WANTUPDATE	_IOW(PESP_IOC_MAGIC,  51, int)
#define PESP_IOCG_PTIMERS 	_IOR(PESP_IOC_MAGIC,  52, ptimer_t)
#define PESP_IOCS_PTIMERS	_IOW(PESP_IOC_MAGIC,  53, ptimer_t)
#define PESP_IOCG_SPY 		_IOR(PESP_IOC_MAGIC,  54, unsigned short int)
#define PESP_IOCS_SPY 		_IOW(PESP_IOC_MAGIC,  55, unsigned short int)

// Andor
#define PESP_IOCG_CSTATUS 	_IOR(PESP_IOC_MAGIC,  56, unsigned long)
#define PESP_IOCS_CSTATUS 	_IOW(PESP_IOC_MAGIC,  57, unsigned long)
#define PESP_IOCG_WTEMP 	_IOR(PESP_IOC_MAGIC,  58, unsigned long)
#define PESP_IOCS_WTEMP 	_IOW(PESP_IOC_MAGIC,  59, unsigned long)
#define PESP_IOCG_TEMP 		_IOR(PESP_IOC_MAGIC,  60, float)
#define PESP_IOCS_TEMP 		_IOW(PESP_IOC_MAGIC,  61, float)
#define PESP_IOCG_FASTTRIGGER 	_IOR(PESP_IOC_MAGIC,  62, bool)
#define PESP_IOCS_FASTTRIGGER 	_IOW(PESP_IOC_MAGIC,  63, bool)
#define PESP_IOCG_VSSPEED 	_IOR(PESP_IOC_MAGIC,  64, unsigned short int)
#define PESP_IOCS_VSSPEED 	_IOW(PESP_IOC_MAGIC,  65, unsigned short int)
#define PESP_IOCG_READOUTTIME	_IOR(PESP_IOC_MAGIC,  66, float)
#define PESP_IOCS_READOUTTIME	_IOW(PESP_IOC_MAGIC,  67, float)

//
#define PESP_IOCG_DOEXTRA 	_IOR(PESP_IOC_MAGIC,  68, short int)
#define PESP_IOCS_DOEXTRA	_IOW(PESP_IOC_MAGIC,  69, short int)


#define PESP_IOC_END 69


#ifndef _HAVE_GC_
#define PESP_IOC_MAXNR PESP_IOC_END
#else

#define PESP_IOC_GC_START (PESP_IOC_END + 1)

#if __D == 1
#warning =========================
#warning Compiling with GC support
#warning =========================
#endif

#define PESP_IOCG_GC_PARAMS	_IOR(PESP_IOC_MAGIC,  PESP_IOC_GC_START+0, gc_in_params_t)
#define PESP_IOCS_GC_PARAMS	_IOW(PESP_IOC_MAGIC,  PESP_IOC_GC_START+1, gc_in_params_t)

#define PESP_IOCG_GC_PROCESS 	_IOR(PESP_IOC_MAGIC,  PESP_IOC_GC_START+2, int)
#define PESP_IOCS_GC_PROCESS	_IOW(PESP_IOC_MAGIC,  PESP_IOC_GC_START+3, int)
#define PESP_IOCG_GC_FASTCALC 	_IOR(PESP_IOC_MAGIC,  PESP_IOC_GC_START+4, two_int_t)
#define PESP_IOCS_GC_FASTCALC	_IOW(PESP_IOC_MAGIC,  PESP_IOC_GC_START+5, two_int_t)
#define PESP_IOCG_GC_ALGORITHM 	_IOR(PESP_IOC_MAGIC,  PESP_IOC_GC_START+6, two_int_t)
#define PESP_IOCS_GC_ALGORITHM	_IOW(PESP_IOC_MAGIC,  PESP_IOC_GC_START+7, two_int_t)
#define PESP_IOCG_GC_ROITYPE 	_IOR(PESP_IOC_MAGIC,  PESP_IOC_GC_START+8, int)
#define PESP_IOCS_GC_ROITYPE	_IOW(PESP_IOC_MAGIC,  PESP_IOC_GC_START+9, int)
#define PESP_IOCG_GC_ROITHRESHOLD 	_IOR(PESP_IOC_MAGIC,  PESP_IOC_GC_START+10, double)
#define PESP_IOCS_GC_ROITHRESHOLD	_IOW(PESP_IOC_MAGIC,  PESP_IOC_GC_START+11, double)
#define PESP_IOCG_GC_ROISEARCHBCKG	_IOR(PESP_IOC_MAGIC,  PESP_IOC_GC_START+12, two_double_t)
#define PESP_IOCS_GC_ROISEARCHBCKG	_IOW(PESP_IOC_MAGIC,  PESP_IOC_GC_START+13, two_double_t)
#define PESP_IOCG_GC_CURTHRES 	_IOR(PESP_IOC_MAGIC,  PESP_IOC_GC_START+14, int16_t)
#define PESP_IOCS_GC_CURTHRES	_IOW(PESP_IOC_MAGIC,  PESP_IOC_GC_START+15, int16_t)

#define PESP_IOC_MAXNR PESP_IOC_GC_START+15
#define PESP_IOC_GC_END PESP_IOC_GC_START+15
#endif

#define PESP_IOC_CLI_START (PESP_IOC_GC_END + 1)
#define PESP_IOCG_CLIREAD	_IOR(PESP_IOC_MAGIC,  PESP_IOC_CLI_START+0, char[512])
#define PESP_IOCS_CLIWRITE	_IOW(PESP_IOC_MAGIC,  PESP_IOC_CLI_START+1, char[512])

#undef PESP_IOC_MAXNR
#define PESP_IOC_MAXNR PESP_IOC_CLI_START+2

#endif //__PESP_IOTCL_H__
