#include "pesp.h"

#define stringify( x ) stringify_literal( x )
#define stringify_literal( x ) # x
const char *Release = stringify( RELEASE );
const char *Repo = stringify( REPO );


#include "h_types_2.h"
#include "h_image_2.h"

//---------------------------------------------------------------------------
/* 
 * module_param(foo, int, 0000)
 * The first param is the parameters name
 * The second param is it's data type
 * The final argument is the permissions bits, 
 * for exposing parameters in sysfs (if non-zero) at a later stage.
 */
long int buf_size = DEFAULT_PESP_BUF_SIZE;
module_param(buf_size, long, S_IRUSR);
MODULE_PARM_DESC(buf_size, "A long integer for buffer size");

//---------------------------------------------------------------------------
unsigned long wr_nr;
unsigned long wr_usecs;
unsigned long rd_usecs;
unsigned long wr_no_ptr_sw;
pid_t	      wr_last_pid_no_ptr_sw;

struct timeval wr_end_time;
struct timeval rd_begin_time[MAX_FD_NR];
struct timeval rd_end_time[MAX_FD_NR];

unsigned long wr_no_upd[MAX_FD_NR];

unsigned long __p;

//---------------------------------------------------------------------------
static struct proc_dir_entry *proc_parent;
static struct proc_dir_entry *proc_dev;
static struct proc_dir_entry *proc_tasks;
static struct proc_dir_entry *proc_pespcli;
static struct proc_dir_entry *proc_gc_parent;
static struct proc_dir_entry *proc_gc_in_params;
static struct proc_dir_entry *proc_rw_gc_process;
static struct proc_dir_entry *proc_rw_gc_qf_flag;
//---------------------------------------------------------------------------
char rclibuf[CLIBUFFSIZE];
char wclibuf[CLIBUFFSIZE];
//---------------------------------------------------------------------------

/* Structure that declares the usual file */
/* access functions */

struct file_operations pesp_fops = {
  .read     = pesp_read,
  .write    = pesp_write,
  .open     = pesp_open,
  .release  = pesp_release,
  .poll	    = pesp_poll,
  #if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,32)
  	.ioctl    = pesp_ioctl,
  #else
  	.unlocked_ioctl = pesp_ioctl,
  #endif
};

/* Global variables of the driver */



h_ccd_t  pespCCD;
h_virt_ccd_t pesp;

stageInfo_t stageInfo;

ccd_point_t OYag;

float pixel2micron;

atomic_t pespcmd;

/* Buffer to store data */
pespBuff_t pespBuff;

#ifdef _WRBUFF_
	char * wr_buf;
#endif


h_metaimg_t dark_buf;

struct class *pesp_class = 0;

dev_t dev_num;
struct cdev cdev;
unsigned baseminor = 0;
int dev_major = 0;

buff_Dt_t buff_Dt[MAX_FD_NR];


#ifdef _HAVE_GC_
	gc_in_params_t gc_in_params;
#endif
//---------------------------------------------------------------------------

int _pespOpened[MAX_FD_NR];

//---------------------------------------------------------------------------
struct timeval ptimers[MAX_FD_NR][5];
//---------------------------------------------------------------------------


#ifdef _WQMEMCPY_
	unsigned long wk_usecs;
	struct work_memcpy *m_work;

	#ifdef _WQWQ_
		struct workqueue_struct *wq = 0;
	#endif

	static void work_memcpy_function(struct work_struct *work){
		int i, ret, data_avail;
	  	struct timeval t0, t1;
	
		struct work_memcpy *work_memcpy_ptr = container_of(work, struct work_memcpy, work_s);
	
	  	do_gettimeofday(&t0);
		//printk(KERN_INFO "[Deferred work]=> PID: %d; NAME: %s\n", current->pid, current->comm);
	
	
	  	data_avail = 0;
	  	wr_nr++;
	
		for (i=1; i<MAX_FD_NR; i++) {
		   if (pespBuff.buff[i].want_update){
		           __p = 50 + i;
		           ret = mutex_trylock(&pespBuff.buff[i].mtx);
		           __p = 100 + i;
		           if (ret != 0) {
		                   // Got the lock!
		                   __p = 200 + i;
		                   #ifdef _WRBUFF_
		                     	memcpy(pespBuff.buff[i].buff, wr_buf, work_memcpy_ptr->count);
					pespBuff.buff[i].wr_count = work_memcpy_ptr->count;
					//printk(KERN_INFO "[Deferred work]=> pespBuff.buff[%d]\n", i);
		                   #endif
		                   __p = 210 + i;
		                   atomic_set(&(pespBuff.buff[i].st), 1);
		                   data_avail = 1;
		                   __p = 230 + i;
		                   mutex_unlock(&pespBuff.buff[i].mtx);
		                   __p = 300 + i;
		           } else {
		                   __p = 400 + i;
		                   wr_no_ptr_sw++;
		                   wr_last_pid_no_ptr_sw = pespBuff.buff[i].pid;
		                   wr_no_upd[i]++;
		                   printk(KERN_ALERT "pesp_write(): not lock, go ahead without ptr switch PID=%d comm=%s\n",
		                           pespBuff.buff[i].pid, pespBuff.buff[i].comm);
		           }
		   }
                   atomic_set(&(buff_Dt[i].st), 1);
		}
	
	  	__p = 5;
	

	        /* finally, awake any writers and return */
	  	if (data_avail) wake_up_interruptible(&(pespBuff.queue));
	
	  	__p = 6;
	
	
	
	  	do_gettimeofday(&t1);
	  	wk_usecs = (t1.tv_sec - t0.tv_sec)*1000000 + (t1.tv_usec - t0.tv_usec);
	
		#ifdef _DBG
			printk(KERN_INFO "[Deferred work]=> DONE. data_avail: %d | %ld usec\n", data_avail, wk_usecs);
		#endif
	
		return;
	}
#endif /* _WQMEMCPY_ */
//---------------------------------------------------------------------------

#ifdef _PROCFS_
	#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,32)
	#else
	
		static int pesp_oproc(struct inode *inode, struct file *file) {
		     return single_open(file, pesp_rproc_dev, NULL);
		}
		static const struct file_operations pesp_proc_fops = {
		     .owner	= THIS_MODULE,
		     .open	= pesp_oproc,
		     .read	= seq_read,
		     .llseek	= seq_lseek,
		     .release	= single_release,
		};

		//--
	
		static int pesptasks_oproc(struct inode *inode, struct file *file) {
		     return single_open(file, pesp_rproc_tasks, NULL);
		}
		static const struct file_operations pesptasks_proc_fops = {
		     .owner	= THIS_MODULE,
		     .open	= pesptasks_oproc,
		     .read	= seq_read,
		     .llseek	= seq_lseek,
		     .release	= single_release,
		};

		//--
	
		static int pesp_gc_in_params_oproc(struct inode *inode, struct file *file) {
		     return single_open(file, pesp_rproc_gc_in_params, NULL);
		}
		static const struct file_operations pesp_gc_in_params_proc_fops = {
		     .owner	= THIS_MODULE,
		     .open	= pesp_gc_in_params_oproc,
		     .read	= seq_read,
		     .llseek	= seq_lseek,
		     .release	= single_release,
		};
		
		//--
	
		static int pesp_gc_process_oproc(struct inode *inode, struct file *file) {
		     return single_open(file, pesp_rproc_gc_process, NULL);
		}
		static const struct file_operations pesp_gc_process_proc_fops = {
		     .owner	= THIS_MODULE,
		     .open	= pesp_gc_process_oproc,
		     .read	= seq_read,
		     .write     = pesp_wproc_gc_process,
		     .llseek	= seq_lseek,
		     .release	= single_release,
		};
		
		//--
	
		static int pesp_gc_qf_flag_oproc(struct inode *inode, struct file *file) {
		     return single_open(file, pesp_rproc_gc_qf_flag, NULL);
		}
		static const struct file_operations pesp_gc_qf_flag_proc_fops = {
		     .owner	= THIS_MODULE,
		     .open	= pesp_gc_qf_flag_oproc,
		     .read	= seq_read,
		     .write     = pesp_wproc_gc_qf_flag,
		     .llseek	= seq_lseek,
		     .release	= single_release,
		};
		
		
		//--
	
		static int pespcli_oproc(struct inode *inode, struct file *file) {
		     return single_open(file, pesp_rproc_cli, NULL);
		}
		static const struct file_operations pespcli_proc_fops = {
		     .owner	= THIS_MODULE,
		     .open	= pespcli_oproc,
		     .read	= seq_read,
		     .write     = pesp_wproc_cli,
		     .llseek	= seq_lseek,
		     .release	= single_release,
		};

	#endif /* LINUX_VERSION_CODE */
#endif /*_PROCFS_ */

//---------------------------------------------------------------------------
int __init pesp_init(void) {
	int result;
	int i,j;
	int dev_maj_min;

	wr_nr = 0;
	wr_no_ptr_sw = 0;

	memset(&pespCCD, 0, sizeof(pespCCD));

	#ifdef _DBG
		printk(KERN_DEBUG "\n ... pesp_init():\n");
		printk(KERN_DEBUG "pesp_init(): buf_size= %ld\n", buf_size);
	#endif

	/* Registering device */
	/* int alloc_chrdev_region(dev_t *dev, unsigned baseminor, unsigned count, const char *name) */
	/* baseminor o first minor qui 0 */
	/* ORA count = 2, voglio i minor number 0 e 1 */

	result = alloc_chrdev_region(&dev_num, baseminor, 2, DEVICE_NAME);
	if (result < 0) {
		printk( KERN_ALERT "pesp_init(): cannot register device\n");
		return result;
	}
	dev_major = MAJOR(dev_num);


	if (buf_size !=0){
		#ifdef _WRBUFF_
	        	wr_buf = (char *)vmalloc(buf_size);
	        	if (!wr_buf)  printk( KERN_EMERG "pesp_init(): cannot have mem for wr\n");
		#endif

		h_data_struct_info_t ds_info = DEFAULT_METAIMG_IMG_STRUCT;
		dark_buf.info = ds_info;
	      	dark_buf.data = (char *)vmalloc(buf_size);
		dark_buf.info.size += buf_size;
	      	if (!dark_buf.data)  {
			printk( KERN_EMERG "pesp_init(): cannot have mem for dark_buf.data\n");
			dark_buf.info.size = 0;
		}
	}


	for (i=0; i<MAX_FD_NR; i++) {

		atomic_set(&(buff_Dt[i].st), 0);
		buff_Dt[i].opened = -1;

		pespBuff.buff[i].opened = -1;
		pespBuff.buff[i].want_update = WANT_UPDATE_DFLT;
		pespBuff.buff[i].pid = 0;
		pespBuff.buff[i].wr_count = 0;
		strncpy(pespBuff.buff[i].comm, "---------------", TASK_COMM_LEN);

		if ( (buf_size !=0) && (i>0) ){
			pespBuff.buff[i].buff = (char *)vmalloc(buf_size);
			if (!pespBuff.buff[i].buff)  printk( KERN_EMERG "pesp_init(): cannot have mem for %d\n", i);
			} else {
				pespBuff.buff[i].buff = NULL;
			}


			atomic_set(&(pespBuff.buff[i].st), 0);
			mutex_init(&pespBuff.buff[i].mtx);

			for (j=0; j<5; j++) {
				ptimers[i][j].tv_sec = 0;
				ptimers[i][j].tv_usec = 0;
			}
			wr_no_upd[i] = 0;
	};

	pespBuff.buff[4].want_update = 0;

	init_waitqueue_head(&(pespBuff.queue));
	atomic_set(&(pespBuff.inited), 1);

	pespCCD.aet.tv_sec = 0;
	pespCCD.aet.tv_usec = 0;

	pesp.darksubtract = false;


	atomic_set( &pespcmd, 0);

	#ifdef _WQMEMCPY_
		m_work = kmalloc(sizeof(*m_work), GFP_KERNEL);
		INIT_WORK(&m_work->work_s, work_memcpy_function);
		#ifdef _WQWQ_
			if (!wq) wq = create_singlethread_workqueue("pesp_wq");
		#endif
	#endif

	 
	
	/*
		populate sysfs entry
	*/
	pesp_class = class_create(THIS_MODULE, DEVICE_NAME);

	/*
		connect the file operations with the cdev
	*/
	cdev_init(&cdev, &pesp_fops);
	cdev.owner = THIS_MODULE;

	/*
		connect the major/minor number to the cdev
	*/
	if (cdev_add(&cdev, dev_num, 2)) {
		printk(KERN_ALERT "pesp_init(): bad cdev on minor:%d\n", baseminor + 0);
		return 1;
	}

	/* seend uevents to udev, so it'll create the /dev node*/
	dev_maj_min = MKDEV(dev_major, baseminor + 0);
	device_create(pesp_class, NULL, dev_maj_min, NULL, "pespImg%d", 0);

	dev_maj_min = MKDEV(dev_major, baseminor + 1);
	device_create(pesp_class, NULL, dev_maj_min, NULL, "pespDt%d", 0);

	printk(KERN_ALERT "pesp_init(): module inserted, Release: %s from Repo: %s\n", Release, Repo); 

	/************************************
	*
	*   PROCFS
	*
	************************************/

	proc_parent = proc_mkdir("pesp",NULL);
	if(!proc_parent) {
		printk(KERN_INFO "Error creating pesp proc entry");
		return -ENOMEM;
	}
	
	proc_gc_parent = proc_mkdir("gc", proc_parent);
	if(!proc_gc_parent) {
		printk(KERN_INFO "Error creating pesp/gc proc entry");
		return -ENOMEM;
	}
	
	#ifdef _PROCFS_
		memset(rclibuf, 0, CLIBUFFSIZE);
		memset(wclibuf, 0, CLIBUFFSIZE);

		#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,32)
			create_proc_read_entry("dev", 0, proc_parent, pesp_rproc_dev, NULL);
			create_proc_read_entry("tasks", 0, proc_parent, pesp_rproc_tasks, NULL);
	
			proc_rw_gc_process = create_proc_entry("process", 0666, proc_gc_parent);
			if(!proc_rw_gc_process) {
	  			printk(KERN_INFO "Error creating pesp/gc/process proc entry");
	  			return -ENOMEM;
	 		}
	
			proc_rw_gc_process->read_proc  = (read_proc_t *)pesp_rproc_gc_process;
			proc_rw_gc_process->write_proc = (write_proc_t*)pesp_wproc_gc_process;
	
			proc_rw_gc_qf_flag = create_proc_entry("qf_flag", 0666, proc_gc_parent);
			if(!proc_rw_gc_qf_flag) {
	  			printk(KERN_INFO "Error creating pesp/gc/qf_flag proc entry");
	  			return -ENOMEM;
	 		}
	
			proc_rw_gc_qf_flag->read_proc  = (read_proc_t *)pesp_rproc_gc_qf_flag;
			proc_rw_gc_qf_flag->write_proc = (write_proc_t*)pesp_wproc_gc_qf_flag;

			proc_pespcli = create_proc_entry("cli", 0666, proc_parent);
			if(!proc_pespcli) {
	  			printk(KERN_INFO "Error creating pesp/cli proc entry");
	  			return -ENOMEM;
	 		}
	
			proc_pespcli->read_proc  = (read_proc_t *)pesp_rproc_cli;
			proc_pespcli->write_proc = (write_proc_t*)pesp_wproc_cli;
		#else
			proc_dev           = proc_create_data("dev", S_IRUGO, proc_parent, &pesp_proc_fops, NULL);
			proc_tasks         = proc_create_data("tasks", S_IRUGO, proc_parent, &pesptasks_proc_fops, NULL);

			proc_gc_in_params  = proc_create_data("gc_in_params", S_IRUGO, proc_gc_parent, &pesp_gc_in_params_proc_fops, NULL);

			proc_rw_gc_process = proc_create_data("process", S_IRUGO | S_IWUGO, proc_gc_parent, &pesp_gc_process_proc_fops, NULL);
			proc_rw_gc_qf_flag = proc_create_data("qf_flag", S_IRUGO | S_IWUGO, proc_gc_parent, &pesp_gc_qf_flag_proc_fops, NULL);

			proc_pespcli       = proc_create_data("cli", S_IRUGO | S_IWUGO, proc_parent, &pespcli_proc_fops, NULL);

		#endif /* LINUX_VERSION_CODE */
	
	#endif /* _PROCFS_ */
	
	return 0;
	
	//fail: 
	pesp_exit(); 
	return result;
}
//---------------------------------------------------------------------------
void __exit pesp_exit(void) {
	int dev_maj_min;
  
	#ifdef _WQMEMCPY_
		flush_work(&m_work->work_s);
		#ifdef _WQWQ_
			if (wq) destroy_workqueue(wq);
		#endif
		kfree(m_work);
	#endif

	#ifdef _PROCFS_
  		remove_proc_entry("process", proc_gc_parent);
		remove_proc_entry("qf_flag", proc_gc_parent);
		remove_proc_entry("gc_in_params", proc_gc_parent);
		remove_proc_entry("gc", proc_parent);
 
		remove_proc_entry("tasks", proc_parent);

		remove_proc_entry("cli", proc_parent);
		remove_proc_entry("dev", proc_parent);

		remove_proc_entry("pesp", NULL);
	#endif

	/* remove the cdev*/
	cdev_del(&cdev);

	/* Freeing the major number */
	unregister_chrdev_region(dev_num, 2);

	dev_maj_min = MKDEV(dev_major, baseminor + 0);
	device_destroy(pesp_class, dev_maj_min);

	dev_maj_min = MKDEV(dev_major, baseminor + 1);
	device_destroy(pesp_class, dev_maj_min);

	class_destroy(pesp_class);

	if (pespBuff.buff[1].buff) {
		vfree(pespBuff.buff[1].buff);
  		printk(KERN_ALERT "pesp_exit(): vfree() 1");
  	};
  	if (pespBuff.buff[2].buff) vfree(pespBuff.buff[2].buff);
  	if (pespBuff.buff[3].buff) vfree(pespBuff.buff[3].buff);
  	if (pespBuff.buff[4].buff) vfree(pespBuff.buff[4].buff);

  	if (dark_buf.data) vfree(dark_buf.data);

  	#ifdef _WRBUFF_
          	if (wr_buf) vfree(wr_buf);
  	#endif

  	printk(KERN_ALERT "pesp_exit(): module removed\n");
}

//---------------------------------------------------------------------------
/* Declaration of the init and exit functions */

EXPORT_SYMBOL(pespBuff);
EXPORT_SYMBOL(pesp_read);
EXPORT_SYMBOL(pesp_poll);
EXPORT_SYMBOL(pesp_ioctl);

module_init(pesp_init);
module_exit(pesp_exit);


