#include "pesp.h"

extern const char *Release;
extern const char *Repo;


extern unsigned long wr_nr;
extern unsigned long wr_usecs;
extern unsigned long rd_usecs;
extern unsigned long wr_no_ptr_sw;
extern pid_t         wr_last_pid_no_ptr_sw;

extern pespBuff_t pespBuff;

extern h_virt_ccd_t pesp;
extern h_ccd_t  pespCCD;

extern struct timeval wr_end_time;
extern struct timeval rd_begin_time[MAX_FD_NR];
extern struct timeval rd_end_time[MAX_FD_NR];

extern unsigned long wr_no_upd[MAX_FD_NR];

extern unsigned long __p;

extern struct timeval ptimers[MAX_FD_NR][5];

extern char rclibuf[CLIBUFFSIZE];

#ifdef _HAVE_GC_
	extern gc_in_params_t gc_in_params;
#endif

#ifdef _WQMEMCPY_
	extern unsigned long wk_usecs;
#endif


extern long int buf_size;

//============================================================================================
typedef struct {
	int integer;
	int decimal;
} k_float_t;

k_float_t float2int2(u32 kf32){
	k_float_t v;

        int s = (kf32 >> 31);
        int e = ((kf32 >> 23) & 0xff);
        int m = (kf32 & 0x7fffff);

	if ((e==0)&&(m==0)) {
		v.integer = 0;
		v.decimal = 0;
		return v;
	}

        u32 m32 = (0x800000 | m);

	// Moltiplico per 100, per poter visualizzare con 2 cifre decimali.
	// 100 = circa 7bit, quindi 24 bit di m32 + 7 = 31 bit < 32bit di m32, quindi no overflow!
	//
	long int k100 = (long int)((m32*100)>>(23 - (e-127))); // k100 = (int)(abs((float)kf32 * 100))

	v.integer = k100 / 100;
	v.decimal = k100 - (v.integer * 100);
	v.integer *= (s?-1:1);

	return v;
}

//============================================================================================
#ifdef _PROCFS_
	#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,32)
	
		//---------------------------------------------------------------------------
		void pesp_proc_offset(char *buf, char **start, off_t *offset, int *len)
		{
		        if (*offset == 0)
		                return;
		        if (*offset >= *len) {
		                /* Not there yet */
		                *offset -= *len;
		                *len = 0;
		        } else {
		                /* We're into the interesting stuff now */
		                *start = buf + *offset;
		                *offset = 0;
		        }
		}
		
		//---------------------------------------------------------------------------
		int pesp_rproc_dev(char *buf, char **start, off_t offset, int count, int *eof, void *data) {
		
		  int i,j;
		  int len = 0;
		  struct timeval btv, etv;
		
			*start = buf;
		
			len += sprintf(buf+len,"\nDevice: pesp\n");
			pesp_proc_offset (buf, start, &offset, &len);
		
			len += sprintf(buf+len,"Release: %s\n", Release);
			pesp_proc_offset (buf, start, &offset, &len);
		
			len += sprintf(buf+len,"Repo: %s\n", Repo);
			pesp_proc_offset (buf, start, &offset, &len);
		
			len += sprintf(buf+len,"\tbuf_size: %lu\n", buf_size);
			pesp_proc_offset (buf, start, &offset, &len);
		
			len += sprintf(buf+len,"\tNumber of writes: %lu\n", wr_nr);
			pesp_proc_offset (buf, start, &offset, &len);
		
			len += sprintf(buf+len,"\tTime of write: %lu [usec]\n", wr_usecs);
			pesp_proc_offset (buf, start, &offset, &len);
		
			#ifdef _WQMEMCPY_
				len += sprintf(buf+len,"\tTime of work: %lu [usec]\n", wk_usecs);
				pesp_proc_offset (buf, start, &offset, &len);
			#endif
		
			len += sprintf(buf+len,"\tProfile write: %lu \n", __p);
			pesp_proc_offset (buf, start, &offset, &len);
		
			len += sprintf(buf+len,"\tTime of read: %lu [usec]\n", rd_usecs);
			pesp_proc_offset (buf, start, &offset, &len);
		
		
			len += sprintf(buf+len,"\tEnd wr time: %lu [sec] %lu [usec]\n", wr_end_time.tv_sec, wr_end_time.tv_usec);
			pesp_proc_offset (buf, start, &offset, &len);
		
			for (i=1; i<MAX_FD_NR; i++){
				btv.tv_sec  = rd_begin_time[i].tv_sec;
				btv.tv_usec = rd_begin_time[i].tv_usec;
				etv.tv_sec  = rd_end_time[i].tv_sec;
				etv.tv_usec = rd_end_time[i].tv_usec;
		
				len += sprintf(buf+len,"\tBegin/End/elaps rd[%d] time: ", i);
				len += sprintf(buf+len,"%lu [sec] %lu [usec] / ", btv.tv_sec, btv.tv_usec);
				len += sprintf(buf+len,"%lu [sec] %lu [usec] / ",  etv.tv_sec, etv.tv_usec);
				
		
		
				if ( btv.tv_sec < etv.tv_sec || (btv.tv_sec == etv.tv_sec && btv.tv_usec < etv.tv_usec) )
					len += sprintf(buf+len,"%lu [usec]",  (etv.tv_sec-btv.tv_sec)*1000000 + (etv.tv_usec-btv.tv_usec));
		
				len += sprintf(buf+len,"\n");
				pesp_proc_offset (buf, start, &offset, &len);
			}
		
			len += sprintf(buf+len,"\tMissing upd: %lu\n", wr_no_ptr_sw);
			pesp_proc_offset (buf, start, &offset, &len);
		
			len += sprintf(buf+len,"\tLast missing upd PID: %d\n", wr_last_pid_no_ptr_sw);
			pesp_proc_offset (buf, start, &offset, &len);
		
		
			for (i=1; i<MAX_FD_NR; i++){
				len += sprintf(buf+len,"\tMissing upd[%d]: %lu\n", i, wr_no_upd[i]);
				pesp_proc_offset (buf, start, &offset, &len);
			};
		
		
			len += sprintf(buf+len,"\tCurrent table:\n\t\tPID\tcomm\t  upd?\n");
			pesp_proc_offset (buf, start, &offset, &len);
		
			for (i=1; i<MAX_FD_NR; i++){
				len += sprintf(buf+len,"\t\t%d\t%s\t%s",
					pespBuff.buff[i].pid,
					pespBuff.buff[i].comm,
					pespBuff.buff[i].want_update?"yes":"no"
				);
				len += sprintf(buf+len,"\t");
				for (j=0; j<5; j++){
					len += sprintf(buf+len,"| %lu %lu", ptimers[i][j].tv_sec, ptimers[i][j].tv_usec);
				}
				len += sprintf(buf+len,"\n");
				pesp_proc_offset (buf, start, &offset, &len);
			}
		
			len += sprintf(buf+len,"\n\t       virt_ccd.width= %d\n", pesp.width);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t      virt_ccd.height= %d\n", pesp.height);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t       virt_ccd.depth= %d\n", pesp.depth);
			pesp_proc_offset (buf, start, &offset, &len);

			#if 0
				// NO float in kernel!
				len += sprintf(buf+len,"\tvirt_ccd.acqframerate= %f\n", pesp.acqframerate);
				pesp_proc_offset (buf, start, &offset, &len);
			#endif

			len += sprintf(buf+len,"\tvirt_ccd.darksubtract= [%d] %s\n", pesp.darksubtract, (pesp.darksubtract?"yes":"no"));
			pesp_proc_offset (buf, start, &offset, &len);
			
			//---------------------------------------------=
			len += sprintf(buf+len,"\n\t            ccd.width= %d\n", pespCCD.width);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t           ccd.height= %d\n", pespCCD.height);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t            ccd.depth= %d\n", pespCCD.depth);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.aet= %d [s] %d [us]\n", pespCCD.aet.tv_sec, pespCCD.aet.tv_usec);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.smd= %c\n", pespCCD.smd);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.amd= %c\n", pespCCD.amd);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.emd= %c\n", pespCCD.emd);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.ads= %d\n", pespCCD.ads);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.atp= %c\n", pespCCD.atp);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.spx= %d\n", pespCCD.spx);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.svo= %d\n", pespCCD.svo);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.svw= %d\n", pespCCD.svw);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.sho= %d\n", pespCCD.sho);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.shw= %d\n", pespCCD.shw);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.ceg= %d\n", pespCCD.ceg);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.ceo= %d\n", pespCCD.ceo);
			pesp_proc_offset (buf, start, &offset, &len);
			len += sprintf(buf+len,"\t              ccd.res= %c\n", pespCCD.res);
			pesp_proc_offset (buf, start, &offset, &len);
		
		        *eof = 1;
		        return len;
		}
		
		
		//---------------------------------------------------------------------------
		int pesp_rproc_tasks(char *buf, char **start, off_t offset, int count, int *eof, void *data) {
		
		  int i,j;
		  int len = 0;
		
			*start = buf;
		
			for (i=1; i<MAX_FD_NR; i++){
				len += sprintf(buf+len,"\t%d\t%s\t%s\t%lu",
					pespBuff.buff[i].pid,
					pespBuff.buff[i].comm,
					pespBuff.buff[i].want_update?"yes":"no",
					wr_no_upd[i]
				);
				for (j=0; j<5; j++){
					len += sprintf(buf+len,"\t %lu.%06lu", ptimers[i][j].tv_sec, ptimers[i][j].tv_usec);
				}
				len += sprintf(buf+len,"\n");
				pesp_proc_offset (buf, start, &offset, &len);
			}
		
		        *eof = 1;
		        return len;
		}
		
		//---------------------------------------------------------------------------
		int pesp_rproc_gc_process(char *buf, char **start, off_t offset, int count, int *eof, void *data){
		
		  int len = 0;
		
		  int v = gc_in_params.gc_roi.process;
		
			*start = buf;
		
			len += sprintf(buf+len,"%d %s\n", v, (v==0?"no":"yes"));
			pesp_proc_offset (buf, start, &offset, &len);
		
		        *eof = 1;
		        return len;
		}
		
		
		//---------------------------------------------------------------------------
		int pesp_rproc_gc_qf_flag(char *buf, char **start, off_t offset, int count, int *eof, void *data){
		
		  int len = 0;
		
		  int v = gc_in_params.gc_roi.qf_flag;
		
			*start = buf;
		
			len += sprintf(buf+len,"%d %s\n", v, (v==0?"no":"yes"));
			pesp_proc_offset (buf, start, &offset, &len);
		
		        *eof = 1;
		        return len;
		}

		//---------------------------------------------------------------------------
                int pesp_rproc_cli(char *buf, char **start, off_t offset, int count, int *eof, void *data){
		
		  int len = 0;
		
			*start = buf;
		
			len += sprintf(buf+len,"%s", rclibuf);
			pesp_proc_offset (buf, start, &offset, &len);
		
		        *eof = 1;
		        return len;
		}


	#else /* LINUX_VERSION_CODE */
		
		//============================================================================================================
		//
		//
		
		int pesp_rproc_dev(struct seq_file *m, void *v) {
		  int i,j;
		  struct timeval btv, etv;
		  k_float_t kfloat;
		
			seq_printf(m, "\n");
			seq_printf(m, "Device : pesp\n");
			seq_printf(m, "Release: %s\n", Release);
			seq_printf(m, "Repo   : %s\n", Repo);
			seq_printf(m, "\n");
			seq_printf(m,"\tbuf_size: %lu\n", buf_size);
		
			seq_printf(m,"\tNumber of writes: %lu\n", wr_nr);
		
			seq_printf(m,"\tTime of write: %lu [usec]\n", wr_usecs);
		
			#ifdef _WQMEMCPY_
				seq_printf(m,"\tTime of work: %lu [usec]\n", wk_usecs);
			#endif
		
			seq_printf(m,"\tProfile write: %lu \n", __p);
		
			seq_printf(m,"\tTime of read: %lu [usec]\n", rd_usecs);
		
		
			seq_printf(m,"\tEnd wr time: %lu [sec] %lu [usec]\n", wr_end_time.tv_sec, wr_end_time.tv_usec);
		
			for (i=1; i<MAX_FD_NR; i++){
				btv.tv_sec  = rd_begin_time[i].tv_sec;
				btv.tv_usec = rd_begin_time[i].tv_usec;
				etv.tv_sec  = rd_end_time[i].tv_sec;
				etv.tv_usec = rd_end_time[i].tv_usec;
		
				seq_printf(m,"\tBegin/End/elaps rd[%d] time: ", i);
				seq_printf(m,"%lu [sec] %lu [usec] / ", btv.tv_sec, btv.tv_usec);
				seq_printf(m,"%lu [sec] %lu [usec] / ",  etv.tv_sec, etv.tv_usec);
				
		
		
				if ( btv.tv_sec < etv.tv_sec || (btv.tv_sec == etv.tv_sec && btv.tv_usec < etv.tv_usec) )
					seq_printf(m,"%lu [usec]",  (etv.tv_sec-btv.tv_sec)*1000000 + (etv.tv_usec-btv.tv_usec));
		
				seq_printf(m,"\n");
			}
		
			seq_printf(m,"\tMissing upd: %lu\n", wr_no_ptr_sw);
		
			seq_printf(m,"\tLast missing upd PID: %d\n", wr_last_pid_no_ptr_sw);
		
		
			for (i=1; i<MAX_FD_NR; i++){
				seq_printf(m,"\tMissing upd[%d]: %lu\n", i, wr_no_upd[i]);
			};
		
		
			seq_printf(m,"\tCurrent table:\n\t\tPID\tcomm\t  upd?\n");
		
			for (i=1; i<MAX_FD_NR; i++){
				seq_printf(m,"\t\t%d\t%s\t%s",
					pespBuff.buff[i].pid,
					pespBuff.buff[i].comm,
					pespBuff.buff[i].want_update?"yes":"no"
				);
				seq_printf(m,"\t");
				for (j=0; j<5; j++){
					seq_printf(m,"| %lu %lu", ptimers[i][j].tv_sec, ptimers[i][j].tv_usec);
				}
				seq_printf(m,"\n");
			}
		
			seq_printf(m,"\n\t       virt_ccd.width= %d\n", pesp.width);
			seq_printf(m,"\t      virt_ccd.height= %d\n", pesp.height);
			seq_printf(m,"\t       virt_ccd.depth= %d\n", pesp.depth);

			#if 0
				// NO float in kernel!
				seq_printf(m,"\tvirt_ccd.acqframerate= %f\n", pesp.acqframerate);
			#endif

			seq_printf(m,"\tvirt_ccd.darksubtract= [%d] %s\n", pesp.darksubtract, (pesp.darksubtract?"yes":"no"));
			
			//---------------------------------------------=
			seq_printf(m,"\n\t            ccd.width      = %d\n", pespCCD.width);
			seq_printf(m,"\t           ccd.height      = %d\n", pespCCD.height);
			seq_printf(m,"\t            ccd.depth      = %d\n", pespCCD.depth);
			seq_printf(m,"\t              ccd.aet      = %d [s] %d [us]\n", pespCCD.aet.tv_sec, pespCCD.aet.tv_usec);
			seq_printf(m,"\t              ccd.smd      = %c\n", pespCCD.smd);
			seq_printf(m,"\t              ccd.amd      = %c\n", pespCCD.amd);
			seq_printf(m,"\t              ccd.emd      = %c\n", pespCCD.emd);
			seq_printf(m,"\t              ccd.ads      = %d\n", pespCCD.ads);
			seq_printf(m,"\t              ccd.atp      = %c\n", pespCCD.atp);
			seq_printf(m,"\t              ccd.spx      = %d\n", pespCCD.spx);
			seq_printf(m,"\t              ccd.spy      = %d\n", pespCCD.spy);
			seq_printf(m,"\t              ccd.svo      = %d\n", pespCCD.svo);
			seq_printf(m,"\t              ccd.svw      = %d\n", pespCCD.svw);
			seq_printf(m,"\t              ccd.sho      = %d\n", pespCCD.sho);
			seq_printf(m,"\t              ccd.shw      = %d\n", pespCCD.shw);
			seq_printf(m,"\t              ccd.ceg      = %d\n", pespCCD.ceg);
			seq_printf(m,"\t              ccd.ceo      = %d\n", pespCCD.ceo);
			seq_printf(m,"\t              ccd.res      = %c\n", pespCCD.res);
			if ( (pespCCD.ccdtype>=CCD_UNKNOWN) && (pespCCD.ccdtype<=A_IKONM934) )
				seq_printf(m,"\t              ccd.ccdtype        =  %s\n", CCDTypeName[pespCCD.ccdtype]);
			else 
				seq_printf(m,"\t              ccd.ccdtype        =  %d\n", pespCCD.ccdtype);
			//...............................................=
			seq_printf(m,"\t              ccd.do_extra       = %d\n", pespCCD.do_extra);
			if (pespCCD.do_extra != 0) {
				switch (pespCCD.ccdtype){
					case A_IKONM934:
						{
						  u32 kf = 0;
						  seq_printf(m,"\t              ccd...cooling      = %d\n", pespCCD.priv_data.andor.cooling);
						  seq_printf(m,"\t              ccd...target_temp  = %d\n", pespCCD.priv_data.andor.target_temp);
						  seq_printf(m,"\t              ccd...cstatus      = %lu\n", pespCCD.priv_data.andor.cstatus);
						  memcpy(&kf, &pespCCD.priv_data.andor.temp, sizeof(u32));
						  kfloat = float2int2(kf);
						  seq_printf(m,"\t              ccd...temp         = %d.%02d\n", kfloat.integer, kfloat.decimal);
						  seq_printf(m,"\t              ccd...fast_trigger = %s\n", (pespCCD.priv_data.andor.fast_trigger?"ON":"OFF"));
						  seq_printf(m,"\t              ccd...vsspeed      = %d\n", pespCCD.priv_data.andor.vsspeed);
						  memcpy(&kf, &pespCCD.priv_data.andor.readouttime, sizeof(u32));
						  kfloat = float2int2(kf);
						  seq_printf(m,"\t              ccd...readouttime  = %d.%02d\n", kfloat.integer, kfloat.decimal);
						}
						break;
					default: break;
				}
			}
		
			return 0;
		}
		
		//---------------------------------------------------------------------------
		int pesp_rproc_tasks (struct seq_file *m, void *v) {
		
		  int i,j;
		
			for (i=1; i<MAX_FD_NR; i++){
				seq_printf(m, "\t%d\t%s\t%s\t%lu", 
					pespBuff.buff[i].pid,
					pespBuff.buff[i].comm,
					pespBuff.buff[i].want_update?"yes":"no",
					wr_no_upd[i]
				);
				for (j=0; j<5; j++){
					seq_printf(m, "\t %lu.%06lu", ptimers[i][j].tv_sec, ptimers[i][j].tv_usec);
				}
				seq_printf(m, "\n");
			}
		
		        return 0;
		}
		
		//---------------------------------------------------------------------------
		int pesp_rproc_gc_in_params (struct seq_file *m, void *v) {


			seq_printf(m, "gc_in_params.gc_img.height                     = %d\n",  gc_in_params.gc_img.height);
			seq_printf(m, "gc_in_params.gc_img.width                      = %d\n",  gc_in_params.gc_img.width);

			#if 0
			// NO double in kernel
			seq_printf(m, "gc_in_params.gc_img.hor_calibration            = %g\n",  gc_in_params.gc_img.hor_calibration);
			seq_printf(m, "gc_in_params.gc_img.ver_calibration            = %g\n",  gc_in_params.gc_img.ver_calibration);
			seq_printf(m, "gc_in_params.gc_img.img_hor_pos_offset         = %g\n",  gc_in_params.gc_img.img_hor_pos_offset);
			seq_printf(m, "gc_in_params.gc_img.img_ver_pos_offset         = %g\n",  gc_in_params.gc_img.img_ver_pos_offset);
			#endif

			seq_printf(m, "gc_in_params.gc_roi.type                       = [%d] ",  gc_in_params.gc_roi.type);
			        switch ( gc_in_params.gc_roi.type){
			          case 0:   seq_printf(m, "disable_roi"); break;
			          case 1:   seq_printf(m, "manual_roi"); break;
			          case 2:   seq_printf(m, "auto_roi"); break;
			          case 3:   seq_printf(m, "auto_manual_roi"); break;
			          default:  seq_printf(m, "unknown");
			        }
			        seq_printf(m, "\n");
			seq_printf(m, "gc_in_params.gc_roi.process                    = [%d] ",  gc_in_params.gc_roi.process);
			        switch ( gc_in_params.gc_roi.process){
			          case 0:   seq_printf(m, "disable process"); break;
			          case 1:   seq_printf(m, "enable process"); break;
			          default:  seq_printf(m, "unknown");
			        }
			        seq_printf(m, "\n");
			seq_printf(m, "gc_in_params.gc_roi.param[0] [hor_left]        = %d\n",  gc_in_params.gc_roi.param[0]);
			seq_printf(m, "gc_in_params.gc_roi.param[1] [ver_up]          = %d\n",  gc_in_params.gc_roi.param[1]);
			seq_printf(m, "gc_in_params.gc_roi.param[2] [hor_right        = %d\n",  gc_in_params.gc_roi.param[2]);
			seq_printf(m, "gc_in_params.gc_roi.param[3] [ver_down]        = %d\n",  gc_in_params.gc_roi.param[3]);

			#if 0
			// NO double in kernel
			seq_printf(m, "gc_in_params.gc_roi.threshold                  = %g\n",  gc_in_params.gc_roi.threshold);
			#endif

			seq_printf(m, "gc_in_params.gc_roi.lp_order                   = %d\n",  gc_in_params.gc_roi.lp_order);
			seq_printf(m, "gc_in_params.gc_roi.hor_grid                   = %d\n",  gc_in_params.gc_roi.hor_grid);
			seq_printf(m, "gc_in_params.gc_roi.ver_grid                   = %d\n",  gc_in_params.gc_roi.ver_grid);

			#if 0
			// NO double in kernel
			seq_printf(m, "gc_in_params.gc_roi.search_background_param[0] = %g\n",  gc_in_params.gc_roi.search_background_param[0]);
			seq_printf(m, "gc_in_params.gc_roi.search_background_param[1] = %g\n",  gc_in_params.gc_roi.search_background_param[1]);
			#endif

			//    int32_t algorithm;                     // RAW_CALC, GAUSS_FIT, GAUSS_FIT_ASYMM, CONFITEOR_FIT,         "ImageProc.h" line 1732
			//    int fast_calc;                      // decimate automatically the image to speedup calculations,    "ImageProc.h" line 174

			#ifndef IMGPROCNEW
			        seq_printf(m, "gc_in_params.algorithm                         = [%ld] ",  gc_in_params.algorithm);
			                switch (gc_in_params.algorithm){
			                  case 0:   seq_printf(m, "RAW_CALC"); break;
			                  case 1:   seq_printf(m, "GAUSS_FIT"); break;
			                  case 2:   seq_printf(m, "GAUSS_FIT_ASYMM"); break;
			                  case 3:   seq_printf(m, "CONFITEOR_FIT"); break;
			                  default:  seq_printf(m, "unknown");
			                }
			                seq_printf(m, "\n");
			        seq_printf(m, "gc_in_params.fast_calc                         = [%d] ",  gc_in_params.fast_calc);
			                switch (gc_in_params.fast_calc){
			                  case 0:   seq_printf(m, "NORM_CALC"); break;
			                  case 1:   seq_printf(m, "FAST_CALC"); break;
			                  default:  seq_printf(m, "unknown");
			                }
			                seq_printf(m, "\n");
			#else /* IMGPROCNEW */
							seq_printf(m, "gc_in_params.cur_thres                         = %d\n",  gc_in_params.cur_thres);
			        	seq_printf(m, "gc_in_params.hor_algorithm                     = [%d] ",  gc_in_params.hor_algorithm);
			                switch (gc_in_params.hor_algorithm){
			                  case 0:   seq_printf(m, "RAW_CALC"); break;
			                  case 1:   seq_printf(m, "GAUSS_FIT"); break;
			                  case 2:   seq_printf(m, "GAUSS_FIT_ASYMM"); break;
			                  case 3:   seq_printf(m, "CONFITEOR_FIT"); break;
			                  default:  seq_printf(m, "unknown");
			                }
			                seq_printf(m, "\n");
					seq_printf(m, "gc_in_params.ver_algorithm                     = [%d] ",  gc_in_params.ver_algorithm);
						switch (gc_in_params.ver_algorithm){
						  case 0:   seq_printf(m, "RAW_CALC"); break;
						  case 1:   seq_printf(m, "GAUSS_FIT"); break;
						  case 2:   seq_printf(m, "GAUSS_FIT_ASYMM"); break;
						  case 3:   seq_printf(m, "CONFITEOR_FIT"); break;
						  default:  seq_printf(m, "unknown");
						}
						seq_printf(m, "\n");
			                seq_printf(m, "\n");
			        	seq_printf(m, "gc_in_params.hor_fast_calc_inc                 = [%d] ",  gc_in_params.hor_fast_calc_inc);
					if (gc_in_params.hor_fast_calc_inc == 0) seq_printf(m, "NORM_CALC");
					else if (gc_in_params.hor_fast_calc_inc > 0) seq_printf(m, "FAST_CALC");
					else seq_printf(m, "unknown");
			                seq_printf(m, "\n");
			        	seq_printf(m, "gc_in_params.ver_fast_calc_inc                 = [%d] ",  gc_in_params.ver_fast_calc_inc);
					if (gc_in_params.ver_fast_calc_inc == 0) seq_printf(m, "NORM_CALC");
					else if (gc_in_params.ver_fast_calc_inc > 0) seq_printf(m, "FAST_CALC");
					else seq_printf(m, "unknown");
			                seq_printf(m, "\n");
		
			                seq_printf(m, "\n");
					//========================================================================================
					// Quality Factor
		
					//h_long qf_roi_1_param[4];
					//h_long qf_roi_2_param[4];
					seq_printf(m, "gc_in_params.gc_roi.qf_flag                             = %d\n",  gc_in_params.gc_roi.qf_flag);
		
					seq_printf(m, "gc_in_params.gc_roi.qf_roi_1_param[0] [hor_left]        = %d\n",  gc_in_params.gc_roi.qf_roi_1_param[0]);
					seq_printf(m, "gc_in_params.gc_roi.qf_roi_1_param[1] [ver_up]          = %d\n",  gc_in_params.gc_roi.qf_roi_1_param[1]);
					seq_printf(m, "gc_in_params.gc_roi.qf_roi_1_param[2] [hor_right        = %d\n",  gc_in_params.gc_roi.qf_roi_1_param[2]);
					seq_printf(m, "gc_in_params.gc_roi.qf_roi_1_param[3] [ver_down]        = %d\n",  gc_in_params.gc_roi.qf_roi_1_param[3]);
			                seq_printf(m, "\n");
		
					seq_printf(m, "gc_in_params.gc_roi.qf_roi_2_param[0] [hor_left]        = %d\n",  gc_in_params.gc_roi.qf_roi_2_param[0]);
					seq_printf(m, "gc_in_params.gc_roi.qf_roi_2_param[1] [ver_up]          = %d\n",  gc_in_params.gc_roi.qf_roi_2_param[1]);
					seq_printf(m, "gc_in_params.gc_roi.qf_roi_2_param[2] [hor_right        = %d\n",  gc_in_params.gc_roi.qf_roi_2_param[2]);
					seq_printf(m, "gc_in_params.gc_roi.qf_roi_2_param[3] [ver_down]        = %d\n",  gc_in_params.gc_roi.qf_roi_2_param[3]);
			                seq_printf(m, "\n");
		
			#endif /* IMGPROCNEW */


		        return 0;
		}

		
		//---------------------------------------------------------------------------
                int pesp_rproc_gc_process(struct seq_file *m, void *v){
		  int w = gc_in_params.gc_roi.process;
		
			seq_printf(m, "%d %s\n", w, (w==0?"no":"yes"));
			return 0;
		}

		//---------------------------------------------------------------------------
                int     pesp_rproc_gc_qf_flag(struct seq_file *m, void *v){
		  int w = gc_in_params.gc_roi.qf_flag;
		
			seq_printf(m, "%d %s\n", w, (w==0?"no":"yes"));
			return 0;
		}

		
		//---------------------------------------------------------------------------
		int pesp_rproc_cli(struct seq_file *m, void *v) {
			seq_printf(m, "|%s|\n", rclibuf);
			return 0;
		}
		
	#endif /* LINUX_VERSION_CODE */
#endif /* _PROCFS_ */

//==================================================================
//AAA
