
#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>

#include "pesp_ioctl.h"


void usage(){
	  printf("\n");
	  printf("Usage: setGC GCparameter value\n");
	  printf("      int32_t width, height\n");
	  printf("      double hor_calibration, ver_calibration\n");
	  printf("      int type 0 = disable_roi,  1 = manual_roi, 2 = auto_roi, 3 = auto_manual_roi\n");
	  printf("      int process 0 = disable process, 1 = enable process\n");
	  printf("      int32_t param: hor_left ver_up hor_right ver_down\n");
	  printf("      double threshold\n");
	  printf("      int cur_thres\n");
	  printf("      int32_t lp_order\n");
	  printf("      int32_t hor_grid, ver_grid\n");
	  printf("      double search_background_param: steps derivative\n");
	  printf("      int hor/ver_fast_calc h,v 0 = normal, >1 fast\n");
	  printf("      int32_t algorithm 0 = RAW_CALC,        1 = GAUSS_FIT\n");
	  printf("                     2 = GAUSS_FIT_ASYMM, 3 = CONFITEOR_FIT\n");
	  printf("\n");
	  printf("      int32_t qf1param: hor_left ver_up hor_right ver_down\n");
	  printf("      int32_t qf2param: hor_left ver_up hor_right ver_down\n");
	  printf("\n");
	  exit(-1);
}

int main(int argc, char *argv[]){
  int hamaBuf_fd = 0;
  gc_in_params_t gc_in_params;
  int retval;

        char *value;
        const char *delim = ",";


	if (argc < 3) usage();

	hamaBuf_fd = open("/dev/pespImg0", O_RDWR);
        if (hamaBuf_fd == -1) {
                perror("");
                return -1;
        }
        printf("hamaBuf_fd = %d\n", hamaBuf_fd);


/*********************************
//.......from ImageProc.h
#define RAW_CALC          0
//.......from Fit.h
#define GAUSS_FIT         1
#define GAUSS_FIT_ASYMM   2
#define CONFITEOR_FIT     3


typedef struct _gc_in_params {
    gc_img_t gc_img;
    gc_roi_t gc_roi;
    int32_t algorithm;                     // RAW_CALC, GAUSS_FIT, GAUSS_FIT_ASYMM, CONFITEOR_FIT,         "ImageProc.h" line 1732
    int fast_calc;                      // decimate automatically the image to speedup calculations,    "ImageProc.h" line 174
} gc_in_params_t;


typedef struct _gc_img{
    unsigned char *imgptr;      // puntatore all'immagine
    int32_t height;
    int32_t width;
    double hor_calibration, ver_calibration; // number of pixels per mm
} gc_img_t;

typedef struct _gc_roi{
    int    type;        // 0 = disable_roi,  1 = manual_roi, 2 = auto_roi, 3 = auto_manual_roi
    int    process;     // 0 = disable process, 1 = enable process
    int32_t   param[4];    // hor_left, ver_up, hor_right, ver_down
    double threshold;
    int32_t   lp_order;    // order of the low pass filter (for automatic ROI detection)
    int32_t   hor_grid;    // number of horizontal cells used to find automatically the ROI
    int32_t   ver_grid;    // number of vertical cells used to find automatically the ROI
    double search_background_param[2];  // [0]: number of steps to search the background level
                                        // [1]: derivative coeff (0->1)
} gc_roi_t;


**********************************/

	if ( strcmp(argv[1], "width") == 0 ) {
		int width;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &width);
			gc_in_params.gc_img.width = width;
			printf("Writing %s %d\n", argv[1], width);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else 
	if ( strcmp(argv[1], "height") == 0 ) {
		int height;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &height);
			gc_in_params.gc_img.height = height;
			printf("Writing %s %d\n", argv[1], height);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	}  else
	if ( strcmp(argv[1], "hor_calibration") == 0 ) {
		float hor_calibration;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%f", &hor_calibration);
			gc_in_params.gc_img.hor_calibration = (double)hor_calibration;
			printf("Writing %s %g\n", argv[1], hor_calibration);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	}  else
	if ( strcmp(argv[1], "ver_calibration") == 0 ) {
		float ver_calibration;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%g", &ver_calibration);
			gc_in_params.gc_img.ver_calibration = (double)ver_calibration;
			printf("Writing %s %g\n", argv[1], ver_calibration);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	}  else
	if ( strcmp(argv[1], "type") == 0 ) {
		int type;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &type);
			gc_in_params.gc_roi.type = type;
			printf("Writing %s %d\n", argv[1], type);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else 
	if ( strcmp(argv[1], "process") == 0 ) {
		int process;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &process);
			gc_in_params.gc_roi.process = process;
			printf("Writing %s %d\n", argv[1], process);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else 
	if ( strcmp(argv[1], "param") == 0 ) {
		int32_t p1, p2, p3, p4;
			if (argc != 6) usage();
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &p1);
			sscanf(argv[3], "%d", &p2);
			sscanf(argv[4], "%d", &p3);
			sscanf(argv[5], "%d", &p4);
			gc_in_params.gc_roi.param[0] = p1;
			gc_in_params.gc_roi.param[1] = p2;
			gc_in_params.gc_roi.param[2] = p3;
			gc_in_params.gc_roi.param[3] = p4;
			printf("Writing %s %d %d %d %d\n", argv[1], p1, p2, p3, p4 );
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else
	if ( strcmp(argv[1], "threshold") == 0 ) {
		float threshold;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%f", &threshold);
			gc_in_params.gc_roi.threshold = (double)threshold;
			printf("Writing %s %g\n", argv[1], threshold);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else
	if ( strcmp(argv[1], "cur_thres") == 0 ) {
		int threshold;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &threshold);
			gc_in_params.cur_thres = (int)threshold;
			printf("Writing %s %d\n", argv[1], threshold);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else
	if ( strcmp(argv[1], "lp_order") == 0 ) {
		int32_t lp_order;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &lp_order);
			gc_in_params.gc_roi.lp_order = lp_order;
			printf("Writing %s %d\n", argv[1], lp_order);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else 
	if ( strcmp(argv[1], "hor_grid") == 0 ) {
		int32_t hor_grid;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &hor_grid);
			gc_in_params.gc_roi.hor_grid = hor_grid;
			printf("Writing %s %d\n", argv[1], hor_grid);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else 
	if ( strcmp(argv[1], "ver_grid") == 0 ) {
		int32_t ver_grid;
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &ver_grid);
			gc_in_params.gc_roi.ver_grid = ver_grid;
			printf("Writing %s %d\n", argv[1], ver_grid);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else  
	if ( strcmp(argv[1], "search_background_param") == 0 ) {
		float p1, p2;
			if (argc != 4) usage();
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%f", &p1);
			sscanf(argv[3], "%f", &p2);
			gc_in_params.gc_roi.search_background_param[0] = (double)p1;
			gc_in_params.gc_roi.search_background_param[1] = (double)p2;
			printf("Writing %s %f %f\n", argv[1], p1, p2);
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else   
	if ( strcmp(argv[1], "fast_calc") == 0 ) {
		two_int_t fast;
                        value = strtok(argv[2], delim);
                        if (value == NULL) {printf("Error in param\n"); exit(-1);};
                        fast.v[0] = atoi(value);
                        value = strtok(NULL, delim);
                        if (value == NULL) {printf("Error in param\n"); exit(-1);};
                        fast.v[1] = atoi(value);
 			printf("Writing %d,%d\n", fast.v[0], fast.v[1]);
                        retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_FASTCALC, &fast);
                        printf("retval = %d\n", retval);
	} else
	if ( strcmp(argv[1], "algorithm") == 0 ) {
		two_int_t algorithm;
                        value = strtok(argv[2], delim);
                        if (value == NULL) {printf("Error in param\n"); exit(-1);};
                        algorithm.v[0] = atoi(value);
                        value = strtok(NULL, delim);
                        if (value == NULL) {printf("Error in param\n"); exit(-1);};
                        algorithm.v[1] = atoi(value);
 			printf("Writing %d,%d\n", algorithm.v[0], algorithm.v[1]);
                        retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_ALGORITHM, &algorithm);
                        printf("retval = %d\n", retval);
	} else
        //========================================================================================
        // Quality Factor

        //h_long qf_roi_1_param[4];
        //h_long qf_roi_2_param[4];
	if ( strcmp(argv[1], "qf1param") == 0 ) {
		int32_t p1, p2, p3, p4;
			if (argc != 6) usage();
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &p1);
			sscanf(argv[3], "%d", &p2);
			sscanf(argv[4], "%d", &p3);
			sscanf(argv[5], "%d", &p4);
			gc_in_params.gc_roi.qf_roi_1_param[0] = p1;
			gc_in_params.gc_roi.qf_roi_1_param[1] = p2;
			gc_in_params.gc_roi.qf_roi_1_param[2] = p3;
			gc_in_params.gc_roi.qf_roi_1_param[3] = p4;
			printf("Writing %s %d %d %d %d\n", argv[1], p1, p2, p3, p4 );
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	} else
	if ( strcmp(argv[1], "qf2param") == 0 ) {
		int32_t p1, p2, p3, p4;
			if (argc != 6) usage();
			printf("Read & Write\n");
			retval = ioctl(hamaBuf_fd,PESP_IOCG_GC_PARAMS, &gc_in_params);          
        		printf("Read retval = %d\n", retval);
			sscanf(argv[2], "%d", &p1);
			sscanf(argv[3], "%d", &p2);
			sscanf(argv[4], "%d", &p3);
			sscanf(argv[5], "%d", &p4);
			gc_in_params.gc_roi.qf_roi_2_param[0] = p1;
			gc_in_params.gc_roi.qf_roi_2_param[1] = p2;
			gc_in_params.gc_roi.qf_roi_2_param[2] = p3;
			gc_in_params.gc_roi.qf_roi_2_param[3] = p4;
			printf("Writing %s %d %d %d %d\n", argv[1], p1, p2, p3, p4 );
        		retval = ioctl(hamaBuf_fd, PESP_IOCS_GC_PARAMS, &gc_in_params);          
			printf("retval = %d\n", retval);
	}


    	close(hamaBuf_fd);
}

